#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config


@dataclass(frozen=True)
class SurveyWiFiScanData(DataClassJsonMixin):
    timestamp: int
    frequency: int
    channel: int
    bssid: str
    strength: int
    ssid: Optional[str] = None
    band: Optional[str] = None
    channelWidth: Optional[int] = None
    capabilities: Optional[str] = None
    latitude: Optional[Number] = None
    longitude: Optional[Number] = None
    altitude: Optional[Number] = None
    heading: Optional[Number] = None
    rssi: Optional[Number] = None
