#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from enum import Enum


class CheckListItemType(Enum):
    simple = "simple"
    string = "string"
    enum = "enum"
    files = "files"
    yes_no = "yes_no"
    cell_scan = "cell_scan"
    wifi_scan = "wifi_scan"
    MISSING_ENUM = ""

    @classmethod
    def _missing_(cls, value: object) -> "CheckListItemType":
        return cls.MISSING_ENUM
